/*!
 * @license New BSD License
 * @copyright Copyright (c) 2010, Nokia Corporation
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *   * Neither the name of the Nokia Corporation nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NOKIA CORPORATION BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef GEOENGINE_H
#define GEOENGINE_H

#include <QObject>
#include <qgeocoordinate.h>

class GeoEnginePrivate;

QT_BEGIN_HEADER

QT_BEGIN_NAMESPACE

QTM_USE_NAMESPACE

/*!
    \class GeoEngine
    \brief The GeoEngine class provides object required by other geo services. It hadles online/offline modes and enables device positioning.

    GeoEngine object provides with basic initialization required by other geo services. It requires
    invocation of init function to be initialized. After init function invoked GeoEngine requires main loop to be started.
*/

class GeoEngine : public QObject
{
friend class GeoFinderPrivate;
friend class GeoMapPrivate;
friend class MapWidgetPrivate;
friend class GeoRoadFinderPrivate;

    Q_OBJECT
    Q_PROPERTY(bool isOnline READ isOnline WRITE setOnline NOTIFY onlineEnabled)
    Q_PROPERTY(bool watchDevicePosition READ watchDevicePosition WRITE setWatchDevicePosition NOTIFY watchDevicePositionEnabled)
    Q_PROPERTY(QGeoCoordinate devicePosition READ devicePosition NOTIFY devicePositionChanged)

public:
    /*!
    * Constructor.
    */
    GeoEngine();

    /*!
    * Destructor.
    */
    ~GeoEngine();

    /*!
    * Initialization function with security credentials support.
    *  @param referer identifier of client who is going to use GeoEngine API.
    *  @param token securuty token to validate client.
    * 
    * @return true (initialized successfully) or false (failed to initialize)
    */
    bool init(const QString& referer, const QString& token);

    /*!
    * Returns true if underlying map model is initialized correctly, 
    * otherwise returns false.
    */
    bool isValid() const;

    /*!
     * Returns true if GeoEngine is online, otherwise returns false.
     */
    bool isOnline() const;

    /*!
     * Returns true if device position showing enabled, returns false otherwise.
     */
    bool watchDevicePosition() const;

    /*!
     * Returns current device position.
     */
    QGeoCoordinate devicePosition() const;

signals:
    /*!
     * This signal is emitted when engine switches online/offline.
     */
    void onlineEnabled(bool enabled);

    /*!
     * This signal is emitted when going online failes.
     */
    void onlineFailed();

    /*!
     * This signal is emitted when device position watching enabled.
     */
    void watchDevicePositionEnabled(bool enabled);

    /*!
     * This signal is emitted when the current device position is changed.
     * New position is specified in pos.
     */
    void devicePositionChanged(const QGeoCoordinate &pos);

public slots:
    /*!
     * Set online or offline mode
     * @param enabled True for online mode and false for offline.
     * \attention Wait for onlineEnabled() signal to be sure mode has been changed.
     */
    void setOnline(bool enabled);

    /*!
     * Enables device position showing.
     * @param enabled set true to enable and false to disable.
     */
    void setWatchDevicePosition(bool enabled);

    /*!
     * Configures power-save mode.
     * By default it enables automatically this mode in 2 minues if power-save conditions.
     * @param msec Number of milliseconds before geoengine switches to power-save mode after
     * getting power-save condition. -1 to disable power-save mode.
     */
    void setPowerSaveMode(int msec);

private:
    Q_DISABLE_COPY(GeoEngine)
    Q_DECLARE_PRIVATE(GeoEngine)
    GeoEnginePrivate *d_ptr;
};

QT_END_NAMESPACE

QT_END_HEADER

#endif
