/*!
 * @license New BSD License
 * @copyright Copyright (c) 2010, Nokia Corporation
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *   * Neither the name of the Nokia Corporation nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NOKIA CORPORATION BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef GEOFINDER_H
#define GEOFINDER_H

#include "GeoCriteria.h"
#include "GeoEngine.h"

class GeoFinderPrivate;

QT_BEGIN_HEADER

QT_BEGIN_NAMESPACE

/*!
    \class GeoFinder
    \brief The GeoFinder class provides geo/reversegeo coding

    GeoFinder object provides with geo/reversegeo coding functionality. It requires GeoEngine object to be created and initialized.
    GeoFinder requires invocation of init function to be initialized and main loop to be started.
*/

class GeoFinder : public QObject
{
    Q_OBJECT
    Q_ENUMS(GeoFinderError)
    Q_ENUMS(GeoFinderCategory)

public:

    enum GeoFinderError {
        NoError,                   // Request successful, i.e. no errors
        GenericError,              // Generic error, search failed
        NoResultsError,            // No online or offline results found
        FallbackNoResultsError,    // Fallback
        CriteriaFailedError        // GeoCriteria verification failed
    };

    enum GeoFinderCategory {
        AirlineAccess,
        AmusementPark,
        CarDealer,
        Casino,
        Cinema,
        Company,
        ConcertHall,
        Congress,
        Courthouse,
        CulturalCentre,
        ExhibitionCentre,
        GolfCourse,
        GovernmentOffice,
        HolidayPark,
        Museum,
        Opera,
        ParkingGarage,
        PetrolStation,
        PlaceOfWorship,
        PostOffice,
        RentACarFacility,
        RestArea,
        Restaurant,
        Shop,
        ShoppingCentre,
        Stadium,
        Theatre,
        TouristAttraction,
        TouristInformationCentre,
        University,
        Zoo,
        Library,
        Camping,
        BarDisco,
        Embassy,
        FerryTerminal,
        FrontierCrossing,
        Hospital,
        Hotel,
        ParkingArea,
        Police,
        RailwayStation,
        MetroStation,
        Airport,
        MountainPass,
        MountainPeak,
        Carrepair,
        CashDispenser,
        ParkRecreation,
        Pharmacy,
        Beach,
        BusStation,
        Education,
        ResidentialArea,
        NightClub,
        PublicToilet,
        Laundry,
        TaxiStand,
        RailwayAccess,
        BarsCafes,
        Parking,
        SportOutdoor,
        MetroAccess,
        AutMetroAccess,
        AutMetroStop,
        BelMetroAccess,
        BelMetroStop,
        CzeMetroAccess,
        CzeMetroStop,
        DenMetroAccess,
        DenMetroStop,
        FinMetroAccess,
        FinMetroStop,
        FraMetroAccess,
        FraMetroStop,
        FraRerAccess,
        FraRerStop,
        DeuMetroAccess,
        DeuMetroStop,
        DeuSbahnAccess,
        DeuSbahnStop,
        ItaMetroAccess,
        ItaMetroStop,
        NorMetroAccess,
        NorMetroStop,
        PrtMetroAccess,
        PrtMetroStop,
        EspBarcelonaMetroAccess,
        EspBarcelonaMetroStop,
        EspCercaniasMetroAccess,
        EspCercaniasMetroStop,
        EspMadridMetroAccess,
        EspMadridMetroStop,
        SweMetroAccess,
        SweMetroStop,
        GbrGlasgowMetroAccess,
        GbrGlasgowMetroStop,
        GbrLondonMetroAccess,
        GbrLondonMetroStop,
        All
    };

    /*!
    * Constructor.
    * @param geoEngine Defines pointer to geo engine. geoEngine must remain valid throughout the life of the GeoFinder.
    */
    GeoFinder(GeoEngine *geoEngine);

    /*!
    * Destructor.
    */
    ~GeoFinder();

    /*!
    * Initialization function. Needed to be invoked before starting using coding APIs.
    * @return true (initialized successfully) or false (failed to initialize)
    */
    bool init();

    /*!
    * Returns a pointer to geo engine.
    */
    GeoEngine* engine() const;

    /*!
    * Begins structured geocoding search. This is asynchronious API.
    * @param country Defines country code (FIN).
    * @param state Optional. Defines state.
    * @param city Defines city name.
    * @param street Defines street name.
    * @param houseNumber Optional. Defines number of the building.
    * @param language Defines the language of the geocoding.
             Must be specified as 3 letter language code.
    * @param numOfResults Defines maximum number of results expected to be returned.
    * @param data Defines pointer to user data.
    * @return true (started successfully) or false (failed to start)
    */
    bool beginGeoCoding(QString country, QString state, QString city, QString street,
                        QString houseNumber, QString language, int numOfResults, void* data);

    /*!
    * Begins one box geocoding search. This is asynchronious API.
    * @param oneBoxString Defines text for search.
    * @param hintCoord Defines hint position.
    * @param language Defines the language of the geocoding.
             Must be specified as 3 letter language code.
    * @param numOfResults Defines maximum number of results expected to be returned.
    * @param data Defines pointer to user data.
    * @return true (started successfully) or false (failed to start)
    */
    bool beginGeoCoding(QString oneBoxString, QGeoCoordinate hintCoord, 
                        QString language, int numOfResults, void* data);

    /*!
    * Begins reverse geocoding search. This is asynchronious API.
    * @param geoCoord Defines position for search.
    * @param data Defines pointer to user data.
    * @param language [Optional] Defines the language of the reverse geocoding.
             Must be specified as 3 letter language code.
    * @param criteria [Optional] Defines critera to accept results.
    * @return true (started successfully) or false (failed to start).
    */
    bool beginReverseGeoCoding(QGeoCoordinate geoCoord, void* data, 
        QString language = "", const GeoCriteria& criteria = GeoCriteria());

    /*!
    * Begins POI geocoding search. This is asynchronious API.
    * @param placeName Defines the POI name.
    * @param category Defines the category to be searched for.
    * @param geoCoord Defines geo position of the vicinity.
    * @param radius Defines minimum search-radius for POIs in meters.
    *        Results may be found outside of this radius.
    * @param language Defines the language of the search.
             Must be specified as 3 letter language code.
    * @param numOfResults Defines maximum number of results expected to be returned.
    * @param data Defines pointer to user data.
    * @return true (started successfully) or false (failed to start)
    */
    bool beginPoiSearch(QString placeName, GeoFinderCategory category, QGeoCoordinate geoCoord,
                        int radius, QString language, int numOfResults, void* data);

    /*!
    * Begins a combination of POI- and address-geocoding search. This is asynchronious API.
    * @param placeName Defines a POI and address combination.
    * @param category Defines the category to be searched for.
    * @param geoCoord Defines geo position of the vicinity.
    * @param radius Defines minimum search-radius for placeName in meters.
    *        Results may be found outside of this radius.
    * @param language Defines the language of the search.
             Must be specified as 3 letter language code.
    * @param numOfResults Defines maximum number of results expected to be returned.
    * @param data Defines pointer to user data.
    * @return true (started successfully) or false (failed to start)
    */
    bool beginCombinedSearch(QString placeName, GeoFinderCategory category, QGeoCoordinate geoCoord,
                             int radius, QString language, int numOfResults, void* data);

    /*!
    * Cancel ongoing coding.
    * @return true or false.
    */
    bool cancelCoding();

    /*!
    * Sets a fallback mode (enabled/disabled).
    * In case offline search returned null results the fallback automaticaly 
    * searches same request online and returns results to the user.
    * @param enabled (true) to enable fallback or (false) to disable fallback.
    */

    void setFallbackMode(bool enabled);

    /*!
    * Returns a fallback mode (enabled/disabled).
    * @return true (fallback mode enabled) or false (fallback mode disabled).
    */
    bool fallbackMode() const;

signals:
    /*!
    * Emitted when coding is started.
    */
    void onStarted(quint32 numOfResults, void* data);

    /*!
    * Emitted when result is generated. Could be emitted several times if several results generated.
    */
    void onResult(const GeoLocationInfo& locInfo, void* data);

    /*!
    * Emitted when coding is finished.
    */
    void onFinished(GeoFinder::GeoFinderError error, void* data);

private:
    Q_DISABLE_COPY(GeoFinder)
    GeoFinderPrivate *f;
};

QT_END_NAMESPACE

QT_END_HEADER

#endif
