/*!
 * @license New BSD License
 * @copyright Copyright (c) 2010, Nokia Corporation
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *   * Neither the name of the Nokia Corporation nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NOKIA CORPORATION BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef GEOLOCATIONINFO_H
#define GEOLOCATIONINFO_H

#include <QObject>
#include <qgeocoordinate.h>

class GeoLocationInfoPrivate;

QT_BEGIN_HEADER

QT_BEGIN_NAMESPACE

QTM_USE_NAMESPACE

/*!
    \class GeoLocationInfo
    \brief The GeoLocationInfo class provides location information generated by geo services

    GeoLocation object provides with location information defined by address fields.
*/

class GeoLocationInfo
{

public:

    /*!
    * Constructor.
    */
    GeoLocationInfo();

    /*!
    * Copy constructor.
    * @param other GeoLocationInfo.
    */
    GeoLocationInfo(const GeoLocationInfo &other);

    /*!
    * Destructor.
    */
    ~GeoLocationInfo();

    enum AddressField
    {
        // Address
        COUNTRY_CODE,      // Country codes, named as in ISO 3166 alpha-3
        COUNTRY_NAME,      // Country name
        STATE_NAME,        // Name of federal state, Bundesland
        STATE_CODE,        // Code of federal state, Bundesland
        PROVINCE_NAME,     // Province name
        COUNTY_NAME,       // County name
        CITY_NAME,         // City name
        DISTRICT_NAME,     // District name
        POSTAL_CODE,       // Postal code
        TOWNSHIP_NAME,     // Township name
        NEIGHBORHOOD_NAME, // Name of neighborhood within the city.
        STREET_NAME,       // Street name
        HOUSE_NUMBER,      // House number (can incl. characters)
        CONTINENT_NAME,    // Continent name
        TYPE_NAME,         // Type of address (city, street, etc.)
        POPULATION,        // Population living at this "address".

        // Building
        BUILDING_NAME,  // Building name
        BUILDING_FLOOR, // Building floor
        BUILDING_ROOM,  // Building room
        BUILDING_ZONE,  // Building zone

        // Place
        PLACE_NAME,         // Name of the place
        PLACE_PRE_NAME,     // Some prefix, for example: "Results nearby."
        PLACE_POST_NAME,    // Similar to prefix.
        PLACE_TYPE,         // The type of the place (premium, adplace, recommendation, etc.)
        PLACE_CATEGORY,     // The name of the category.
        PLACE_CATEGORY_ID,  // The identifier (number as a string) of the category.
        PLACE_DESCRIPTION,  // Short description of the place. Also used as input for place search.
        PLACE_PHONE_NUMBER, // Phone number of the place
        PLACE_URL,          // URL belonging to the place
        PLACE_URL_ID,       // Premium POI URL Id
        PLACE_NODE_ID,      // Premium POI Node Id (aka PlaceId)
        PLACE_ADVERTISEMENT_STRING, // Advertisement string (should be a single line)

        FIELD_LAST         // Marker for last entry
    };

    /*!
    * Check for availability of address field.
    * @param aField AddressField.
    */
    bool has(AddressField aField) const;

    /*!
    * Returns address field.
    * @param aField AddressField.
    */
    QString get(AddressField aField) const;

    /*!
    * Sets value to address field.
    * @param aField AddressField.
    * @param aString value.
    */
    void set(AddressField aField, QString aString);

    /*!
    * Returns geo coordinate of location.
    */
    const QGeoCoordinate& getCoordinate() const;

    /*!
    * Sets geo coordinate for location.
    */
    void setCoordinate(const QGeoCoordinate &coord);

private:
    Q_DECLARE_PRIVATE(GeoLocationInfo)
    GeoLocationInfoPrivate *d_ptr;

};

QT_END_NAMESPACE

QT_END_HEADER

#endif
