/*!
 * @license New BSD License
 * @copyright Copyright (c) 2010, Nokia Corporation
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *   * Neither the name of the Nokia Corporation nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NOKIA CORPORATION BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef GEOMAPTILE_H
#define GEOMAPTILE_H

#include <QObject>
#include <QSize>
#include <QImage>
#include <QX11Info>
#include "GeoEngine.h"

class GeoMapTilePrivate;

QT_BEGIN_HEADER

QT_BEGIN_NAMESPACE

/*!
    \class GeoMapTile
    \brief The GeoMapTile class provides map tiles fetching

    GeoMapTile object provides with map tiles fetching functionality. It requires GeoEngine object to be created and initialized.
    GeoMapTile requires invocation of init function to be initialized and main loop to be started.
*/

class GeoMapTile : public QObject
{
    Q_OBJECT

public:

    /*!
    * Constructor.
    * @param tileSize Defines size of the map tile.
    */
    GeoMapTile(QSize tileSize, GeoEngine *geoEngine);

    /*!
    * Destructor. DO NOT forget to destroy object to cache map data into diskcache.
    */
    ~GeoMapTile();

    /*!
    * Returns a pointer to geo engine.
    */
    GeoEngine* engine() const;

    /*!
    * Initialization function. Needed to be invoked once before using GeoMapTile APIs.
    * @return true or false.
    */
    bool init();

    /*!
    * Requests map tile based on scaling parameter. This is asynchronious APIs.
    * @param coord Coordinate.
    * @param scale Defines amount of centimetres in one pixel.
    * @param data Defines pointer to user data
    */
    void getMapTileByScale(QGeoCoordinate coord, int scale, void* data);

    /*!
    * Requests map tile based on zoom parameter. This is asynchronious APIs.
    * @param coord Coordinate.
    * @param zoomLevel Defines zoom level.
    * @param data Defines pointer to user data
    */
    void getMapTileByZoomLevel(QGeoCoordinate coord, int zoomLevel, void* data);

    /*!
    * Converts zoom level value to corresponding zoom scale value.
    * @return the zoom scale value for given zoom level. In case of errors it returns -1.
    * @param zoomLevel value of zoom level to convert
    */
    int zoomLevelToScale(int zoomLevel) const;

    /*!
    * @return QImage requested.
    */
    QImage getImage();

signals:
    /*!
    * Emitted when image is generated.
    * @param data User data included into request.
    */
    void onImageReady(void* data);

private slots:
    void drawingFinishedSlot();
    void mapReInitialized();

private:
    Q_DISABLE_COPY(GeoMapTile)
    GeoMapTilePrivate *d;
};

QT_END_NAMESPACE

QT_END_HEADER

#endif
